/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package queues;

import java.util.Collections;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.message.ChunkAlreadyExistsException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageType;
import org.objectweb.dream.message.MessageTypeImpl;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.queue.SequenceNumberChunk;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This component has a task that periodically pushes a message on its Push
 * output.
 */
public class PushGeneratorImpl extends AbstractComponent
{

  Push                          outPushItf;
  MessageManager                messageManagerItf;
  MessageType                   msgType;
  int[]                         sns     = {0, 1, 2, 3, 4, 5, 7, 8, 9, 10, 11,
      12, 13, 14, 15, 2, 3, 4, 5, 6     };
  protected static final Object LOCK    = new Object();
  int                           counter = 0;

  /**
   * Constructor.
   */
  public PushGeneratorImpl()
  {
    try
    {
      msgType = new MessageTypeImpl(HelloWorldChunk.DEFAULT_NAME,
          HelloWorldChunk.TYPE, SequenceNumberChunk.DEFAULT_NAME,
          SequenceNumberChunk.TYPE);
    }
    catch (ChunkAlreadyExistsException e)
    {
      // Cannot happen
    }
  }

  // -------------------------------------------------------------------------
  // Overriden methods
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#beforeFirstStart(org.objectweb.fractal.api.Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Util.addTask(componentItf, new GeneratorTask("Push generator task1"),
          Collections.EMPTY_MAP);
      Util.addTask(componentItf, new GeneratorTask("Push generator task2"),
          Collections.EMPTY_MAP);
      logger.log(BasicLevel.DEBUG, "tasks added");
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME};
  }

  // ---------------------------------------------------------------------------
  // The task.
  // ---------------------------------------------------------------------------

  private class GeneratorTask extends AbstractTask
  {

    private String message;

    /**
     * Constructor
     */
    public GeneratorTask(String message)
    {
      super(message);
      this.message = message;
    }

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      try
      {
        Message msg = messageManagerItf.createMessage(msgType);
        HelloWorldChunk chunk = (HelloWorldChunk) msg
            .getChunk(HelloWorldChunk.DEFAULT_NAME);
        SequenceNumberChunk snchunk = (SequenceNumberChunk) msg
            .getChunk(SequenceNumberChunk.DEFAULT_NAME);
        synchronized (LOCK)
        {
          if (counter < 20)
          {
            // The 20 first messages are used to test ordering...
            chunk.setMessage("Message " + message);
            snchunk.setSequenceNumber(sns[counter]);
            counter++;
            outPushItf.push(msg, null);
          }
          else
          {
            chunk.setMessage("Message " + message);
            snchunk.setSequenceNumber(counter++);
            outPushItf.push(msg, null);
          }
        }
      }
      catch (Exception e)
      {
        e.printStackTrace();
        return STOP_EXECUTING;
      }
      Thread.sleep(1000);
      return EXECUTE_AGAIN;
    }
  }
}