/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package utobcast;

import java.net.InetAddress;

import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.rmi.registry.Registry;
import org.objectweb.fractal.util.Fractal;

/**
 * The bootstrap to be launched on each site for distributed deployment.
 */
public final class Bootstrap
{

  /**
   * Constructor.
   */
  private Bootstrap()
  {

  }

  /**
   * Entry point.
   * 
   * @param args the name of the registry host.
   * @throws Exception if an error occurs.
   */
  public static void main(final String[] args) throws Exception
  {
    // the RMI class loader does not work if there is no security manager
    System.setSecurityManager(new SecurityManager());

    NamingService ns;
    String canonicalHostName = InetAddress.getLocalHost().getHostName();
    if (args.length == 0 || canonicalHostName.equals(args[0]))
    {
      Registry.createRegistry(Registry.DEFAULT_PORT);
      ns = Registry.getRegistry();
    }
    else
    {
      ns = Registry.getRegistry(args[0]);
    }

    ns.bind(canonicalHostName, Fractal.getBootstrapComponent());
    System.err.println("Server registered under " + canonicalHostName);
  }

}