/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package utobcast.draw;

import java.io.Serializable;

/**
 * This class represents a Draw command.
 */
public class DrawCommand implements Serializable
{
  static final int DRAW  = 1;
  static final int CLEAR = 2;
  int              mode;
  int              x     = 0;
  int              y     = 0;
  int              r     = 0;
  int              g     = 0;
  int              b     = 0;

  DrawCommand(int mode)
  {
    this.mode = mode;
  }

  DrawCommand(int mode, int x, int y, int r, int g, int b)
  {
    this.mode = mode;
    this.x = x;
    this.y = y;
    this.r = r;
    this.g = g;
    this.b = b;
  }

  DrawCommand copy()
  {
    return new DrawCommand(mode, x, y, r, g, b);
  }

  /**
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    StringBuffer ret = new StringBuffer();
    switch (mode)
    {
      case DRAW :
        ret
            .append("DRAW(" + x + ", " + y + ") [" + r + "|" + g + "|" + b
                + "]");
        break;
      case CLEAR :
        ret.append("CLEAR");
        break;
      default :
        return "<undefined>";
    }
    return ret.toString();
  }

}