/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package utobcast.draw;

import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.adl.FactoryFactory;
import org.objectweb.dream.protocol.utobcast.ProcessMembership;
import org.objectweb.dream.util.Util;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.rmi.registry.Registry;
import org.objectweb.fractal.util.Fractal;

import utobcast.TCPProcess;

/**
 * A class that defines a main method to launch the example.
 */
public class Main
{

  /**
   * Starts a new MonitoringPump
   * 
   * @param args command line arguments
   * @throws Exception if an error occurs.
   */
  public static void main(String[] args) throws Exception
  {
    Factory f = FactoryFactory.getFactory();
    Map ctx = new HashMap();

    // Create the composite
    ctx.put("naming-service", Registry.getRegistry("zirconium.inrialpes.fr"));
    Component composite = (Component) f.newComponent("utobcast.draw.Draw", ctx);

    // Start the composite
    Fractal.getLifeCycleController(composite).startFc();

    // Configure process membership

    //Create 5 processes

    TCPProcess[] tcpProcesses = new TCPProcess[3];
    tcpProcesses[0] = new TCPProcess((short) 0, InetAddress.getLocalHost(),
        30000);
    tcpProcesses[1] = new TCPProcess((short) 1, InetAddress.getLocalHost(),
        30001);
    tcpProcesses[2] = new TCPProcess((short) 2, InetAddress.getLocalHost(),
        30002);
    //    tcpProcesses[3] = new TCPProcess((short) 3, InetAddress.getLocalHost(),
    //        30003);
    //    tcpProcesses[4] = new TCPProcess((short) 4, InetAddress.getLocalHost(),
    //        30004);

    ContentController contentController = Fractal
        .getContentController(composite);

    for (int i = 0; i < tcpProcesses.length; i++)
    {
      Component node = Util.getComponentByName(contentController, Integer
          .toString(i));
      ContentController cc = Fractal.getContentController(node);
      ProcessMembership processMembershipItf = (ProcessMembership) Util
          .getComponentByName(cc, "UTOBcastProtocol/ProcessMembership")
          .getFcInterface(ProcessMembership.ITF_NAME);
      processMembershipItf.setLeader(tcpProcesses[0]);
      processMembershipItf.setBackup(tcpProcesses[1]);
      processMembershipItf.setMyself(tcpProcesses[i]);
    }

    // Retrieve the process membership of the leader
    ProcessMembership leaderProcessMembershipItf = (ProcessMembership) Util
        .getComponentByName(contentController,
            "0/UTOBcastProtocol/ProcessMembership").getFcInterface(
            ProcessMembership.ITF_NAME);

    // Retrieve the process membership of the leader
    ProcessMembership backupMembershipItf = (ProcessMembership) Util
        .getComponentByName(contentController,
            "1/UTOBcastProtocol/ProcessMembership").getFcInterface(
            ProcessMembership.ITF_NAME);

    for (int i = 0; i < tcpProcesses.length; i++)
    {
      leaderProcessMembershipItf.addProcess(tcpProcesses[i]);
      backupMembershipItf.addProcess(tcpProcesses[i]);
    }

  }
}