/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.net.Socket;

/**
 * Basic implementation of {@link SocketState}.
 */
public class SocketStateImpl implements SocketState
{
  protected Socket socket;
  protected Object input;
  protected Object output;

  // ---------------------------------------------------------------------------
  // Implementation of the CodecInputOutput interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketState#setSocket(Socket)
   */
  public void setSocket(Socket socket)
  {
    this.socket = socket;
  }

  /**
   * @see org.objectweb.dream.message.codec.CodecInputOutput#getInput()
   */
  public Object getInput() throws IOException
  {
    if (input == null)
    {
      input = socket.getInputStream();
    }
    return input;
  }

  /**
   * @see org.objectweb.dream.message.codec.CodecInputOutput#setInput(Object)
   */
  public void setInput(Object input)
  {
    this.input = input;
  }

  /**
   * @see org.objectweb.dream.message.codec.CodecInputOutput#getOutput()
   */
  public Object getOutput() throws IOException
  {
    if (output == null)
    {
      output = socket.getOutputStream();
    }
    return output;
  }

  /**
   * @see org.objectweb.dream.message.codec.CodecInputOutput#setOutput(Object)
   */
  public void setOutput(Object output)
  {
    this.output = output;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the SocketState interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketState#isClosed()
   */
  public boolean isClosed()
  {
    return socket.isClosed() || !socket.isConnected();
  }

  /**
   * @see SocketState#close()
   */
  public void close()
  {
    if (!socket.isClosed())
    {
      try
      {
        socket.close();
      }
      catch (IOException e)
      {
        // ignore
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Recyclable interface
  // ---------------------------------------------------------------------------

  /**
   * WARNING : The socket state must be previously closed.
   * 
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    socket = null;
    input = null;
    output = null;
  }

}