/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.utobcast;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Implementation of the PendingMessagesDataStructure component. This component
 * stores pending messages. It is accessed by the REP and ACK components.
 */
public class PendingMessagesDataStructureImpl extends AbstractComponent
    implements
      Push,
      ReceiveAck,
      Pull
{

  // ---------------------------------------------------------------------------
  // Fields of this class
  // ---------------------------------------------------------------------------

  /**
   * The commonly used name to refer to the <code>pendingMessageOutItf</code>
   * interface.
   */
  public static final String  PENDING_MESSAGES_OUT_ITF_NAME = "pending-messages-out";

  /** The interface to retrieve process membership information. */
  protected ProcessMembership processMembershipItf;

  /** The interface to manage message lifecycle. */
  protected MessageManager    messageManagerItf;

  // ---------------------------------------------------------------------------
  // Constructor
  // ---------------------------------------------------------------------------

  /**
   * Constructor.
   */
  public PendingMessagesDataStructureImpl()
  {
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    // TODO Auto-generated method stub
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Pull interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Pull#pull(java.util.Map)
   */
  public Message pull(Map context) throws PullException
  {
    // TODO Auto-generated method stub
    return null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the ReceiveAck interface
  // ---------------------------------------------------------------------------

  /**
   * @see ReceiveAck#receiveAck(Message)
   */
  public void receiveAck(Message message)
  {
    // TODO Auto-generated method stub
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(ProcessMembership.ITF_NAME))
    {
      processMembershipItf = (ProcessMembership) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }

  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{ProcessMembership.ITF_NAME, MessageManager.ITF_NAME};
  }

}