/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.Message;

/**
 * This interface defines methods that must be implemented by a buffer for
 * removing messages from its begining or from its end.
 */
public interface BufferRemoveFirstLast
{
  /** The commonly used name to refer to this interface. */
  String ITF_NAME = "buffer-remove-first-last";

  /**
   * Returns the first message in this buffer. If no message is available, this
   * method blocks.
   * 
   * @return the first message in this list.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   */
  Message getFirst() throws InterruptedException;

  /**
   * Removes and returns the first message from this buffer. If no message is
   * available, this method blocks.
   * 
   * @return the first message from this buffer.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   */
  Message removeFirst() throws InterruptedException;

  /**
   * Returns the last message in this buffer. If no message is available, this
   * method blocks.
   * 
   * @return the last message in this list.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   */
  Message getLast() throws InterruptedException;

  /**
   * Removes and returns the last message from this buffer. If no message is
   * available, this method blocks.
   * 
   * @return the last message from this buffer.
   * @throws InterruptedException if it is interrupted while waiting for a
   *           message to be added.
   */
  Message removeLast() throws InterruptedException;
}