/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.LinkedList;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.util.EmptyStringArray;

/**
 * The ListAddRemoveFirstLastFastImpl class implements the
 * {@link org.objectweb.dream.queue.List},
 * {@link org.objectweb.dream.queue.ListAddFirstLast}, and
 * {@link org.objectweb.dream.queue.ListRemoveFirstLast}interfaces using a
 * {@link java.util.LinkedList}.
 * <p>
 * <i>Note: </i>None of the methods in this linked list are synchronized.
 * <p>
 */
public class ListAddRemoveFirstLastImpl extends AbstractComponent
    implements
      List,
      ListAddFirstLast,
      ListRemoveFirstLast
{

  private LinkedList list = new LinkedList();

  //---------------------------------------------------------------------------
  // Implementation of the List interface.
  // ---------------------------------------------------------------------------

  /**
   * @see List#add(Object)
   */
  public void add(Object o)
  {
    list.add(o);
  }

  /**
   * @see List#remove()
   */
  public Object remove()
  {
    return list.removeFirst();
  }

  /**
   * @see List#isEmpty()
   */
  public boolean isEmpty()
  {
    return list.isEmpty();
  }

  //---------------------------------------------------------------------------
  // Implementation of the ListAddFirstLast interface.
  // ---------------------------------------------------------------------------

  /**
   * @see ListAddFirstLast#addLast(Object)
   */
  public void addLast(Object o)
  {
    list.addLast(o);
  }

  /**
   * @see ListAddFirstLast#addFirst(Object)
   */
  public void addFirst(Object o)
  {
    list.addFirst(o);
  }

  //---------------------------------------------------------------------------
  // Implementation of the ListRemoveFirstLast interface.
  // ---------------------------------------------------------------------------

  /**
   * @see ListRemoveFirstLast#getLast()
   */
  public Object getLast()
  {
    return list.getLast();
  }

  /**
   * @see ListRemoveFirstLast#removeLast()
   */
  public Object removeLast()
  {
    return list.removeLast();
  }

  /**
   * @see ListRemoveFirstLast#getFirst()
   */
  public Object getFirst()
  {
    return list.getFirst();
  }

  /**
   * @see ListRemoveFirstLast#removeFirst()
   */
  public Object removeFirst()
  {
    return list.removeFirst();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}