/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.ChunkTypeImpl;

/**
 * Chunk interface for messages that have to be sequentially ordered.
 */
public interface SequenceNumberChunk
{

  /** The default name for chunks of this type. */
  String    DEFAULT_NAME = "sequenceNumberChunk";

  /** The Chunk type of chunks that implements this interface. */
  ChunkType TYPE         = new ChunkTypeImpl(SequenceNumberChunk.class,
                             SequenceNumberChunkImpl.class);

  /**
   * Returns the sequence number of this message.
   * 
   * @return the sequence number of this message.
   */
  long getSequenceNumber();

  /**
   * Sets the sequence number of this message.
   * 
   * @param sn the sequence number to set.
   */
  void setSequenceNumber(long sn);

}